using System;
using ILOG.Diagrammer;
using ILOG.Diagrammer.Graphic;
using System.Collections;

using SimulationEngine;
using System.Collections.Generic;
using System.Drawing;

namespace CustomSymbols
{
    public partial class Diagram1 : Group
    {
        private Rect currentScreenSelection = null;

        private bool _enableSelection = true;
        public bool EnableSelection
        {
            get
            {
                return _enableSelection;
            }
        }

        public Diagram1()
        {
            InitializeComponent();	// required by the designer
        }

        private ArrayList PassedSymbols = new ArrayList();
        private ArrayList highlightedSymbols = new ArrayList();

        private SimulationEngine.SimulationController _simuController = null;

        public void Init(SimulationController simuController)
        {
            _simuController = simuController;

            IEnumerator<GraphicObject> it = this.Objects.GetEnumerator();
            while (it.MoveNext())
            {
                if (it.Current as UserInterface != null)
                {
                    ((UserInterface)(it.Current)).SetSimuController(_simuController);
                }
                else if (it.Current as WebService != null)
                {
                    ((WebService)(it.Current)).SetSimuController(_simuController);
                }
            }

            _simuController.CurrentScreenChanged += new SimulationController.CurrentScreenChangedHandler(SimuController_CurrentScreenChanged);
            _simuController.ScreenScopeChanged += new SimulationController.ScreenScopeChangedHandler(SimuController_ScreenScopeChanged);
            _simuController.SimulationStateChanged += new SimulationController.SimulationStateChangedHandler(SimuController_SimulationStateChanged);
        }

        private void SimuController_SimulationStateChanged(object sender, bool started)
        {
            _enableSelection = started;

            if(!started)
            {
                this.RevertFilling();
                this.RemoveSelection();
            }
        }

        private void SimuController_ScreenScopeChanged(object sender, ArrayList screens)
        {
            //Highlight the ones which are not yet highlighted
            foreach(string screenName in screens)
            {
                if(!highlightedSymbols.Contains(screenName))
                {
                    highlightedSymbols.Add(screenName);
                    UserSymbol symbol = GetSymbolByName(screenName);

                    if(symbol as UserInterface != null)
                    {
                        ((UserInterface)symbol).Highlight();
                    }
                    else if(symbol as WebService != null)
                    {
                        ((WebService)symbol).Highlight();
                    }
                }
            }

            //Unhighlight the onces which are no more highlighted
            ArrayList unhighlightMe = new ArrayList();
            foreach(string symbolName in highlightedSymbols)
            {
                if(!screens.Contains(symbolName))
                {
                    unhighlightMe.Add(symbolName);
                }
            }

            foreach(string s in unhighlightMe)
            {
                highlightedSymbols.Remove(s);

                UserSymbol symbol = GetSymbolByName(s);

                if(symbol as UserInterface != null)
                {
                    ((UserInterface)symbol).Unhighlight();
                }
                else if(symbol as WebService != null)
                {
                    ((WebService)symbol).Unhighlight();
                }
            }
        }

        private UserSymbol GetSymbolByName(string symbolName)
        {
            UserSymbol match = null;

            IEnumerator<GraphicObject> it = this.Objects.GetEnumerator();
            UserSymbol someSymbol = null;
            while(it.MoveNext())
            {
                someSymbol = it.Current as UserSymbol;
                if(someSymbol != null)
                {
                    if(someSymbol.Name.ToLower().Equals(symbolName.ToLower()))
                    {
                        match = someSymbol;
                    }
                }
            }

            return match;
        }

        private void SimuController_CurrentScreenChanged(object sender, string currentScreen)
        {
            if(currentScreen == null) throw new ArgumentNullException("SimuController_CurrentScreenChanged(object sender, string currentScreen)");

            AddSelection(GetSymbolByName(currentScreen));

            if(this._simuController.Started)
            {
                UpdateFilling();

                if(this.PassedSymbols.Contains(GetSymbolByName(currentScreen)))
                {
                    return;
                }
                else
                    this.PassedSymbols.Add(GetSymbolByName(currentScreen));
            }
        }

        private void UpdateFilling()
        {
            IEnumerator it = this.PassedSymbols.GetEnumerator();
            while(it.MoveNext())
            {
                if(it.Current as UserInterface != null)
                {
                    ((UserInterface)(it.Current)).FillSymbol();
                }
                else if(it.Current as WebService != null)
                {
                    ((WebService)(it.Current)).FillSymbol();
                }
            }
        }

        private void RevertFilling()
        {
            IEnumerator it = this.PassedSymbols.GetEnumerator();
            while(it.MoveNext())
            {
                if(it.Current as UserInterface != null)
                {
                    ((UserInterface)(it.Current)).UnFillSymbol();
                }
                else if(it.Current as WebService != null)
                {
                    ((WebService)(it.Current)).UnFillSymbol();
                }
            }

            this.PassedSymbols.Clear();
        }

        #region Event Handlers
        private void SuccessfulButton_MouseClick(object sender, ObjectMouseEventArgs e)
        {            
            //Shape s = this.GetListOfComplaintsService as Shape;
            //this.GetListOfComplaintsService.Width = 300;
            //s.Fill = new SolidFill(System.Drawing.Color.FromArgb(((int)(((byte)(0)))), ((int)(((byte)(192)))), ((int)(((byte)(192))))));
        }

        
        #endregion

        public void AddSelection(UserSymbol symbol2select)
        {
            this.RemoveSelection();

            if(symbol2select == null) return;

            Rectangle2D geometry = symbol2select.Rectangle;

            // New Width & Left Position
            float newWidth = geometry.Width * (float)(1.1);
            float widthDelta = newWidth - geometry.Width;
            float offsetX = (widthDelta / 2) * -1;
            geometry.Width = newWidth;

            // New Height & Top Position
            float newHeight = geometry.Height * (float)(1.2);
            float heightDelta = newHeight - geometry.Height;
            float offsetY = (heightDelta / 2) * -1;
            geometry.Height = newHeight;

            geometry.Offset(offsetX, offsetY);

            //Create selection rect and add to diagram
            ILOG.Diagrammer.Graphic.Rect selectionRect = new ILOG.Diagrammer.Graphic.Rect(geometry);
            selectionRect.Fill = new SolidFill(Color.Red);
            selectionRect.Stroke = new Stroke(Color.Red);
            selectionRect.Opacity = (float)0.5;
            selectionRect.Stroke.Width = 4;
            selectionRect.Visibility = ILOG.Diagrammer.Visibility.Visible;
            
            currentScreenSelection = selectionRect;
            
            this.Objects.Add(selectionRect);

            this.SetChildZOrder(selectionRect, this.GetChildZOrder(symbol2select) - 1);
        }

        public void RemoveSelection()
        {
            if(this.currentScreenSelection != null)
            {
                this.Objects.Remove(this.currentScreenSelection);
            }
        }
    }
}
