﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

using ProcessView;

using WorkflowLibraryTemplates;

namespace JIGSAWApplication
{
	public partial class ProcessViewPane
	{
        private bool _builderVisible = false;
        private bool _showLibrary = false;
        private LibraryMode _libraryMode;

        private InterfaceLibraryTemplate _interfaceLib;
        private ServiceLibraryTemplate _serviceLib;

        public event DrillDownHandler DrillDownEvent;

        private void OnDrillDown(DrillDownEventArgs args)
        {
            if(DrillDownEvent != null)
            {
                DrillDownEvent(this, args);
            }
        }

		public ProcessViewPane()
		{
			this.InitializeComponent();

            // Init Libraries
            if(this.libraryDock.Children.Count > 0)
            {
                _serviceLib = (ServiceLibraryTemplate)this.libraryDock.Children[0];
            }

            _interfaceLib = new InterfaceLibraryTemplate();
            _interfaceLib.Margin = new Thickness(0, 0, 4, 0);

            //this.processToolbar.tbtAddConnection.Click += new RoutedEventHandler(tbtAddConnection_Click);
            //this.processToolbar.tbtAddTask.Click += new RoutedEventHandler(tbtAddTask_Click);
            //this.processToolbar.tbtPan.Click += new RoutedEventHandler(tbtPan_Click);
            //this.processToolbar.tbtZoomIn.Click += new RoutedEventHandler(tbtZoomIn_Click);
            //this.processToolbar.tbtZoomOut.Click += new RoutedEventHandler(tbtZoomOut_Click);

            this._interfaceLib.BtAddUIScreen.Click += new RoutedEventHandler(BtAddUIScreen_Click);
            this._interfaceLib.AddScreenEvent += new EventHandler(InterfaceLib_AddScreenEvent);
            this._serviceLib.BtAddService.Click += new RoutedEventHandler(BtAddService_Click);
            this._serviceLib.AddServiceEvent += new EventHandler(ServiceLib_AddServiceEvent);

            this.processCanvasHost.Canvas.DrillDownEvent += new DrillDownHandler(Canvas_DrillDownEvent);

            this.ShowBuilder(false);
		}

        void InterfaceLib_AddScreenEvent(object sender, EventArgs e)
        {
            this.workflowCanvasHost.Canvas.AddScreenLibraryInteractor();
        }

        void ServiceLib_AddServiceEvent(object sender, EventArgs e)
        {
            this.workflowCanvasHost.Canvas.AddServiceLibraryInteractor();
        }

        void Canvas_DrillDownEvent(object sender, DrillDownEventArgs args)
        {
            OnDrillDown(args);
        }

        void BtAddService_Click(object sender, RoutedEventArgs e)
        {
            this.workflowCanvasHost.Canvas.AddService(this._serviceLib.CurrentLabel);
        }

        void BtAddUIScreen_Click(object sender, RoutedEventArgs e)
        {
            this.workflowCanvasHost.Canvas.AddUIScreen(this._interfaceLib.CurrentLabel);
        }

        public void ShowBuilder(bool showBuilder)
        {
            this.BeginInit();

            if(showBuilder)
            {
                //Show Builder
                this.processViewInnerFirstRowDef.Height = new GridLength(2, GridUnitType.Star);
                this.processViewInnerSecondRowDef.Height = new GridLength(1, GridUnitType.Star);

                this.workflowCanvasHost.Margin = new Thickness(8, 8, 8, 4);
                this.processCanvasHost.Margin = new Thickness(8, 0, 8, 8);

                //this.processCanvasHost.Canvas.ShowAll(900, 250);
            }
            else
            {
                //Hide Builder
                this.processViewInnerFirstRowDef.Height = new GridLength(0, GridUnitType.Star);
                this.processViewInnerSecondRowDef.Height = new GridLength(1, GridUnitType.Star);

                this.processCanvasHost.Margin = new Thickness(8, 8, 8, 8);

                //this.processCanvasHost.Canvas.ClearSelection();

                //this.processCanvasHost.Canvas.ShowAll(900, 800);

                
            }
            this.EndInit();
            _builderVisible = showBuilder;
        }

        public void ToggleLibrary(bool showLibrary)
        {
            _showLibrary = showLibrary;

            if(_showLibrary)
            {
                this.processViewFirstColDef.Width = new GridLength(380, GridUnitType.Pixel);
                this.processViewSecondColDef.Width = new GridLength(1, GridUnitType.Star);
            }
            else
            {
                this.processViewFirstColDef.Width = new GridLength(0, GridUnitType.Star);
                this.processViewSecondColDef.Width = new GridLength(1, GridUnitType.Star);
            }
            
        }

        public void ChooseLibrary(LibraryMode mode)
        {
            if(_libraryMode == mode)
            {
                ToggleLibrary(!_showLibrary);
            }
            else
            {
                SetLibrary(mode);
                ToggleLibrary(true);
            }
        }

        private void SetLibrary(LibraryMode mode)
        {
            if(mode == this._libraryMode)
                return;

            this.libraryDock.Children.Clear();

            switch(mode)
            {
                case LibraryMode.ServiceLib:
                    this.libraryDock.Children.Add(this._serviceLib);
                    
                    break;

                case LibraryMode.UILib:
                    this.libraryDock.Children.Add(this._interfaceLib);

                    break;

                default:
                    break;
            }

            _libraryMode = mode;
        }

        //void tbtZoomOut_Click(object sender, RoutedEventArgs e)
        //{
        //    this.processCanvasHost.Canvas.InteractorSelectionMode(
        //        ProcessView.ProcessCanvas.CanvasInteractionMode.ZoomOut,
        //        !this.processToolbar.tbtZoomOut.IsChecked.Value);
        //}

        //void tbtZoomIn_Click(object sender, RoutedEventArgs e)
        //{
        //    this.processCanvasHost.Canvas.InteractorSelectionMode(
        //        ProcessView.ProcessCanvas.CanvasInteractionMode.ZoomIn,
        //        !this.processToolbar.tbtZoomIn.IsChecked.Value);
        //}

        //void tbtPan_Click(object sender, RoutedEventArgs e)
        //{
        //    this.processCanvasHost.Canvas.InteractorSelectionMode(
        //         ProcessView.ProcessCanvas.CanvasInteractionMode.Pan,
        //         !this.processToolbar.tbtPan.IsChecked.Value);
        //}

        //void tbtAddTask_Click(object sender, RoutedEventArgs e)
        //{
        //    this.processCanvasHost.Canvas.InteractorSelectionMode(
        //        ProcessView.ProcessCanvas.CanvasInteractionMode.AddTask,
        //        !this.processToolbar.tbtAddTask.IsChecked.Value);
        //}

        //void tbtAddConnection_Click(object sender, RoutedEventArgs e)
        //{
        //    this.processCanvasHost.Canvas.InteractorSelectionMode(
        //        ProcessView.ProcessCanvas.CanvasInteractionMode.AddConnection,
        //        !this.processToolbar.tbtAddConnection.IsChecked.Value);
        //}
	}
}