﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

using ProcessView;

namespace JIGSAWApplication
{
	public partial class ProcessView_NavigationPanel
	{
        public event SwitchViewHandler ViewSwitchedEvent;

        private ProcessCanvas _processCanvas;

        public ProcessView_NavigationPanel()
        {
            this.InitializeComponent();

            AttachObservers();

            this.navigator.Enabled = false;
        }

        public void SetProcessCanvas(ProcessCanvas canvas)
        {
            if(canvas != null)
            {
                _processCanvas = canvas;
                _processCanvas.InteractorResetEvent += new InteractorChangedHandler(_processCanvas_InteractorResetEvent);
                _processCanvas.SelectionChangedEvent += new SelectionChangedHandler(_processCanvas_SelectionChangedEvent);
            }
        }

        private void AttachObservers()
        {
            //this.processView.Click += new RoutedEventHandler(processView_Click);
            //this.testingView.Click += new RoutedEventHandler(testingView_Click);
            //this.workflowView.Click += new RoutedEventHandler(workflowView_Click);
            this.navigator.SliderChangedEvent += new ViewSlider.UserControl1.SliderChangedHandler(navigator_SliderChangedEvent);

            this.toolbarProcessView.btAddConnection.Click += new RoutedEventHandler(btAddConnection_Click);
            this.toolbarProcessView.btAddTask.Click += new RoutedEventHandler(btAddTask_Click);
            this.toolbarProcessView.btAddSystemTask.Click += new RoutedEventHandler(btAddSystemTask_Click);
            this.toolbarProcessView.btAddStartState.Click += new RoutedEventHandler(btAddStartState_Click);
            this.toolbarProcessView.btAddEndState.Click += new RoutedEventHandler(btAddEndState_Click);
            this.toolbarProcessView.btServiceActivation.Click += new RoutedEventHandler(btServiceActivation_Click);
            this.toolbarProcessView.btAddDecision.Click += new RoutedEventHandler(btAddDecision_Click);
            //this.toolbarProcessView.btDelete.Click += new RoutedEventHandler(btDelete_Click);

            this.navigator.OriginalSliderPos = ViewSlider.SliderPos.ProcessPos;
        }

        void navigator_SliderChangedEvent(object sender, ViewSlider.SliderPos sliderPos)
        {
            switch(sliderPos)
            {
                case ViewSlider.SliderPos.ProcessPos:
                    OnViewSwitched(JIGSAWMode.ProcessView);
                    break;

                case ViewSlider.SliderPos.WorkflowPos:
                    OnViewSwitched(JIGSAWMode.WorkflowComposer);
                    break;

                case ViewSlider.SliderPos.TestPos:
                    OnViewSwitched(JIGSAWMode.Testing);
                    break;

                default:
                    break;
            }
        }

        void btDelete_Click(object sender, RoutedEventArgs e)
        {
            //Delete the current selection

            if(_processCanvas != null)
            {
                this._processCanvas.DeleteSelection();
            }
        }

        void _processCanvas_SelectionChangedEvent(object sender, bool drillDownAllowed)
        {
            this.navigator.Enabled = drillDownAllowed;
        }

        void _processCanvas_InteractorResetEvent(object sender, ILOG.Diagrammer.Windows.Forms.ViewInteractor newInteractor)
        {
            this.toolbarProcessView.ResetToggleButtons();
        }

        void btAddDecision_Click(object sender, RoutedEventArgs e)
        {
            _processCanvas.InteractorSelectionMode(ProcessCanvas.CanvasInteractionMode.AddDecision, false);
        }

        void btServiceActivation_Click(object sender, RoutedEventArgs e)
        {
            //TODO: Show errors in process canvas
        }

        void btAddSystemTask_Click(object sender, RoutedEventArgs e)
        {
            _processCanvas.InteractorSelectionMode(ProcessCanvas.CanvasInteractionMode.AddSystemTask, false);
        }

        void btAddEndState_Click(object sender, RoutedEventArgs e)
        {
            _processCanvas.InteractorSelectionMode(ProcessCanvas.CanvasInteractionMode.AddEndPoint, false);
        }

        void btAddStartState_Click(object sender, RoutedEventArgs e)
        {
            _processCanvas.InteractorSelectionMode(ProcessCanvas.CanvasInteractionMode.AddStartPoint, false);
        }

        void btAddTask_Click(object sender, RoutedEventArgs e)
        {
            _processCanvas.InteractorSelectionMode(ProcessCanvas.CanvasInteractionMode.AddTask, false);
        }

        void btAddConnection_Click(object sender, RoutedEventArgs e)
        {
            _processCanvas.InteractorSelectionMode(ProcessCanvas.CanvasInteractionMode.AddConnection, false);
        }

        #region Event Handlers - Main View Toggle Buttons
        private void workflowView_Click(object sender, RoutedEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.WorkflowComposer);
        }

        private void testingView_Click(object sender, RoutedEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.Testing);
        }

        private void processView_Click(object sender, RoutedEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.ProcessView);
        }
        void ImgTest_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.Testing);
        }

        void ImgTask_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.WorkflowComposer);
        }

        void ImgProcess_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.ProcessView);
        }
        #endregion

        public ProcessView_MyNewToolbarTrayControl ToolBar
        {
            get
            {
                return (ProcessView_MyNewToolbarTrayControl)this.toolbarProcessView;
            }
        }

        private void OnViewSwitched(JIGSAWMode mode)
        {
            if((mode == JIGSAWMode.WorkflowComposer) || (mode == JIGSAWMode.Testing))
            {
                this._processCanvas.DrillDownSelectedTask();
            }

            if(ViewSwitchedEvent != null)
            {
                ViewSwitchedEvent(this, mode);
            }
        }
	}
}