﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;

using ProcessView;

namespace JIGSAWApplication
{
	public partial class TaskView_NavigationPanel
	{
        public event SwitchViewHandler ViewSwitchedEvent;
        public event ShowLibraryHandler ShowLibrarySelectedEvent;

        private WorkflowCanvas _workflowCanvas = null;

        public TaskView_NavigationPanel()
        {
            this.InitializeComponent();

            AttachObservers();
        }

        public void SetWorkflowCanvas(WorkflowCanvas canvas)
        {
            if(canvas != null)
            {
                _workflowCanvas = canvas;
                _workflowCanvas.InteractorResetEvent += new InteractorChangedHandler(_workflowCanvas_InteractorResetEvent);
            }
        }

        void _workflowCanvas_InteractorResetEvent(object sender, ILOG.Diagrammer.Windows.Forms.ViewInteractor newInteractor)
        {
            this.toolbarTaskView.ResetToggleButtons();
        }

        private void AttachObservers()
        {
            //this.processView.Click += new RoutedEventHandler(processView_Click);
            //this.testingView.Click += new RoutedEventHandler(testingView_Click);
            //this.workflowView.Click += new RoutedEventHandler(workflowView_Click);
            //this.navigator.ImgProcess.MouseUp += new System.Windows.Input.MouseButtonEventHandler(ImgProcess_MouseUp);
            //this.navigator.ImgTask.MouseUp += new System.Windows.Input.MouseButtonEventHandler(ImgTask_MouseUp);
            //this.navigator.ImgTest.MouseUp += new System.Windows.Input.MouseButtonEventHandler(ImgTest_MouseUp);
            this.navigator.SliderChangedEvent += new ViewSlider.UserControl1.SliderChangedHandler(navigator_SliderChangedEvent);

            this.toolbarTaskView.btAddDataConnection.Click += new RoutedEventHandler(btAddDataConnection_Click);
            this.toolbarTaskView.btAddInterface.Click += new RoutedEventHandler(btAddInterface_Click);
            this.toolbarTaskView.btAddNavConnection.Click += new RoutedEventHandler(btAddNavConnection_Click);
            this.toolbarTaskView.btAddService.Click += new RoutedEventHandler(btAddService_Click);
            //this.toolbarTaskView.btDelete.Click += new RoutedEventHandler(btDelete_Click);
            this.toolbarTaskView.btEdit.Click += new RoutedEventHandler(btEdit_Click);
            this.toolbarTaskView.btServiceActivation.Click += new RoutedEventHandler(btServiceActivation_Click);

            this.navigator.OriginalSliderPos = ViewSlider.SliderPos.WorkflowPos;
        }

        void navigator_SliderChangedEvent(object sender, ViewSlider.SliderPos sliderPos)
        {
            switch(sliderPos)
            {
                case ViewSlider.SliderPos.ProcessPos:
                    OnViewSwitched(JIGSAWMode.ProcessView);
                    break;

                case ViewSlider.SliderPos.WorkflowPos:
                    OnViewSwitched(JIGSAWMode.WorkflowComposer);
                    break;

                case ViewSlider.SliderPos.TestPos:
                    OnViewSwitched(JIGSAWMode.Testing);
                    break;

                default:
                    break;
            }
        }

        private bool _showErrors = false;
        void btServiceActivation_Click(object sender, RoutedEventArgs e)
        {
            //TODO: Show error indicator in the workflow canvas
            if(_workflowCanvas != null)
            {
                this._showErrors = !this._showErrors;
                
                _workflowCanvas.IndicateErrors(this._showErrors);
            }
        }

        void btEdit_Click(object sender, RoutedEventArgs e)
        {
            // TODO: Show "Not Yet Implemented Dialog"
        }

        void btDelete_Click(object sender, RoutedEventArgs e)
        {
            //Delete the current selection

            if(_workflowCanvas != null)
            {
                _workflowCanvas.DeleteSelection();
            }
        }

        void btAddService_Click(object sender, RoutedEventArgs e)
        {
            this.OnLibrarySelected(LibraryMode.ServiceLib);    
        }

        void btAddNavConnection_Click(object sender, RoutedEventArgs e)
        {
            this._workflowCanvas.AddDataFlowConnection();
        }

        void btAddInterface_Click(object sender, RoutedEventArgs e)
        {
            this.OnLibrarySelected(LibraryMode.UILib);
        }

        void btAddDataConnection_Click(object sender, RoutedEventArgs e)
        {
            this._workflowCanvas.AddNavFlowConnection();    
        }

        private void workflowView_Click(object sender, RoutedEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.WorkflowComposer);
        }

        private void testingView_Click(object sender, RoutedEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.Testing);
        }

        private void processView_Click(object sender, RoutedEventArgs e)
        {
            OnViewSwitched(JIGSAWMode.ProcessView);
        }

        public TaskView_MyNewToolbarTrayControl ToolBar
        {
            get
            {
                return (TaskView_MyNewToolbarTrayControl)this.toolbarTaskView;
            }
        }

        public void ResetLibraryToggleButtons()
        {
            this.ToolBar.ResetLibraryToggleButtons();
        }

        private void OnViewSwitched(JIGSAWMode mode)
        {
            if(ViewSwitchedEvent != null)
            {
                ViewSwitchedEvent(this, mode);
            }
        }
        
        private void OnLibrarySelected(LibraryMode mode)
        {
            if(ShowLibrarySelectedEvent != null)
            {
                ShowLibrarySelectedEvent(this, mode);
            }
        }
	}
}