﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Input;
using System.ComponentModel;
using System.Xml;

using SimulationEngine;
using ProcessScreenRepository;
using System.Collections.Generic;
using System.Windows.Media.Imaging;

namespace JIGSAWApplication
{
    public delegate void SwitchViewHandler(object sender, JIGSAWMode mode);
    public delegate void ShowLibraryHandler(object sender, LibraryMode mode);

    public partial class Window1
    {
        private MyNavigationPaneControl myNavigationPaneControl = null;
        private DockPanel dockPanel = null;
        private MouseButtonEventHandler mbEventHandler = null;
        private DependencyPropertyChangedEventHandler visibilityEventHandler = null;

        private JIGSAWMode _appMode = JIGSAWMode.Testing;

        private UIElement _testPanel = null;
        private ProcessViewPane _processPanel = null;

        private navigationPanel _navPanelSimulation = null;
        private TaskView_NavigationPanel _navPanelTaskView = null;
        private ProcessView_NavigationPanel _navPanelProcessView = null;

        private Border toolbarBorder2 = new Border();
        private Border mainBorder2 = new Border();
        private Border toolbarBorder3 = new Border();

        //private Image _legendProcess = new Image();
        //private Image _legendWorkflow = new Image();

        private bool _isSwitchingMode = false;

	    public Window1()
        {
            this.InitializeComponent();

	        myNavigationPaneControl = new MyNavigationPaneControl();

            simuController = new SimulationController(this.SimuPane);

            this.bpmnCanvasHost.Canvas.SetSimuController(simuController);

            InitPanels();

            AttachObservers();

            SetAppMode(JIGSAWMode.ProcessView);
        }

        #region Properties
        public JIGSAWMode AppMode
        {
            get
            {
                return _appMode;
            }
        }

        private SimulationController simuController = null;
        public SimulationController SimuController
        {
            get
            {
                return simuController;
            }
        }
        #endregion

        private void SetAppMode(JIGSAWMode newMode)
        {
            if(this.AppMode == newMode) return;
  
            //TODO: SwitchOff Layout Updates
            this.mainContentPanel.BeginInit();

            this.mainContentPanel.Children.Clear();

            switch(newMode)
            {
                case JIGSAWMode.ProcessView:
                    //this.myTestingMenuItem.Header = "Testing";
                    this.Title = "JIGSAW - Composite Application Builder (Process View)";

                    this._processPanel.ShowBuilder(false);
                    this._processPanel.ToggleLibrary(false);
                    this._processPanel.processCanvasHost.Canvas.ReadOnly = false;
                    this._processPanel.processCanvasHost.Canvas.ShowLegend = true;

                    this.mainContentPanel.Children.Add(toolbarBorder2);
                    DockPanel.SetDock(toolbarBorder2, Dock.Top);

                    this.mainContentPanel.Children.Add(mainBorder2);
                    DockPanel.SetDock(mainBorder2, Dock.Top);

                    DockPanel.SetZIndex(toolbarBorder2, DockPanel.GetZIndex(mainBorder2) + 1);

                    /*
                    if(this._processPanel.processCanvasHost.Canvas.HasDrillDownSelection)
                    {
                        this._navPanelProcessView.navigator.Enabled = true;
                    }
                    else
                    {
                        this._navPanelProcessView.navigator.Enabled = false;
                    }
                    */

                    this._appMode = JIGSAWMode.ProcessView;
                    break;

                case JIGSAWMode.WorkflowComposer:
                    //this.myTestingMenuItem.Header = "Testing";
                    this.Title = "JIGSAW - Composite Application Builder (Workflow View)";

                    this._processPanel.ShowBuilder(true);
                    this._navPanelTaskView.ResetLibraryToggleButtons();
                    this._processPanel.processCanvasHost.Canvas.ReadOnly = true;
                    this._processPanel.processCanvasHost.Canvas.ShowLegend = false;

                    this.mainContentPanel.Children.Add(toolbarBorder3);
                    DockPanel.SetDock(toolbarBorder3, Dock.Top);

                    this.mainContentPanel.Children.Add(mainBorder2);
                    DockPanel.SetDock(mainBorder2, Dock.Top);

                    

                    this._appMode = JIGSAWMode.WorkflowComposer;
                    break;

                case JIGSAWMode.Testing:
                    //this.myTestingMenuItem.Header = "Composing";
                    this.Title = "JIGSAW - Composite Application Builder (Component View)";

                    /*
                    this.mainContentBorder.Child = null;
                    this.toolbarBorder.Child = this._navPanelSimulation;
                    this.mainContentBorder.Child = this._testPanel;
                     */
                    
                    this.mainContentPanel.Children.Add(this.toolbarBorder);
                    DockPanel.SetDock(this.toolbarBorder, Dock.Top);

                    this.mainContentPanel.Children.Add(this.contentBorder);
                    DockPanel.SetDock(this.contentBorder, Dock.Top);

                    this._appMode = JIGSAWMode.Testing;
                    break;

                default:
                    break;
            }

            //TODO: Switch On Layout Updates
            this.mainContentPanel.EndInit();
            this._isSwitchingMode = true;
        }

        #region Properties Button & Panel animation
        private void PropertiesButton_Clicked(object sender, RoutedEventArgs e)
        {
            if (myNavigationPaneControl.Parent == null)
            {
                Grid grid = (Grid)this.PropertiesButton.Parent;
                grid.Children.Add(myNavigationPaneControl);
                myNavigationPaneControl.SetValue(Grid.ColumnSpanProperty, 2);
                myNavigationPaneControl.SetValue(Grid.RowSpanProperty, 1);
                myNavigationPaneControl.HorizontalAlignment = HorizontalAlignment.Right;
                myNavigationPaneControl.VerticalAlignment = VerticalAlignment.Top;
                myNavigationPaneControl.Visibility = Visibility.Hidden;
                if (dockPanel == null)
                {
                    dockPanel = (DockPanel)(((Border)grid.Parent).Parent);
                }
                if (mbEventHandler == null)
                {
                    mbEventHandler = new MouseButtonEventHandler(dockPanel_PreviewMouseLeftButtonUp);
                }
                if (visibilityEventHandler == null)
                {
                    visibilityEventHandler = new DependencyPropertyChangedEventHandler(myNavigationPaneControl_IsVisibleChanged);
                }
            }
            if (myNavigationPaneControl.Visibility == Visibility.Hidden)
            {
                this.PropertiesButton.SetValue(Button.BackgroundProperty, (Brush)App.Current.Resources["MyOrangeSolidBrush"]);
                myNavigationPaneControl.Margin = new Thickness(0, 30, 22, 0);
                myNavigationPaneControl.Visibility = Visibility.Visible;
                dockPanel.PreviewMouseLeftButtonUp += mbEventHandler;
                myNavigationPaneControl.IsVisibleChanged += visibilityEventHandler;
            }
            else
            {
                hideNavigationPane();
            }

        }
        void myNavigationPaneControl_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            if (myNavigationPaneControl.Visibility == Visibility.Hidden)
            {
                hideNavigationPane();
            }
        }

        void dockPanel_PreviewMouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (e.OriginalSource != this.PropertiesButton && e.Source != myNavigationPaneControl)
            {
                if (myNavigationPaneControl.Visibility == Visibility.Visible)
                {
                    hideNavigationPane();
                }
            }
        }

        private void hideNavigationPane()
        {
            this.PropertiesButton.SetValue(Button.BackgroundProperty, (Brush)App.Current.Resources["MyBrightBlueSolidBrush2"]);
            myNavigationPaneControl.Visibility = Visibility.Hidden;
            dockPanel.PreviewMouseLeftButtonUp -= mbEventHandler;
            myNavigationPaneControl.IsVisibleChanged -= visibilityEventHandler;
        }
        #endregion

        private void AttachObservers()
        {
            // Window Handlers
            this.Closing += new CancelEventHandler(Window1_Closing);

            // Toolbar Handlers
            this._navPanelSimulation.ViewSwitchedEvent += new SwitchViewHandler(NavPanelSimulation_ViewSwitchedEvent);
            this._navPanelTaskView.ViewSwitchedEvent += new SwitchViewHandler(NavPanelTaskView_ViewSwitchedEvent);
            this._navPanelProcessView.ViewSwitchedEvent += new SwitchViewHandler(NavPanelProcessView_ViewSwitchedEvent);

            this._navPanelTaskView.ShowLibrarySelectedEvent += new ShowLibraryHandler(_navPanelTaskView_ShowLibrarySelectedEvent);

            this._processPanel.DrillDownEvent += new ProcessView.DrillDownHandler(_processPanel_DrillDownEvent);

            this.LayoutUpdated += new EventHandler(Window1_LayoutUpdated);

            this.simuController.CurrentScreenChanged += new SimulationController.CurrentScreenChangedHandler(simuController_CurrentScreenChanged);

            this._navPanelSimulation.ToolBar.btServiceD.Click += new RoutedEventHandler(btServiceD_Click);
        }

        void btServiceD_Click(object sender, RoutedEventArgs e)
        {
            this.SimuPane.ToggleDataDependency();
        }

        void simuController_CurrentScreenChanged(object sender, string newScreen)
        {
            this.OnLoaded1_BeginStoryboard1.Storyboard.Begin(this);
        }

        

        #region Event Handlers        
        private void Window1_LayoutUpdated(object sender, EventArgs e)
        {
            if(this._isSwitchingMode)
            {
                switch(this._appMode)
                {
                    case JIGSAWMode.ProcessView:

                        this._processPanel.processCanvasHost.Canvas.ShowAll();
                        this._processPanel.processCanvasHost.Canvas.ZoomOut();
                        break;

                    case JIGSAWMode.WorkflowComposer:

                        this._processPanel.workflowCanvasHost.Canvas.ShowAll();
                        this._processPanel.processCanvasHost.Canvas.ShowAll();
                        break;

                    case JIGSAWMode.Testing:

                        this.bpmnCanvasHost.Canvas.ShowAll();
                        break;

                    default:
                        break;
                }

                this._isSwitchingMode = false;
            }
        }

        private void _processPanel_DrillDownEvent(object sender, ProcessView.DrillDownEventArgs args)
        {           
            SetAppMode(JIGSAWMode.WorkflowComposer);
        }
        
        private void _navPanelTaskView_ShowLibrarySelectedEvent(object sender, LibraryMode mode)
        {
            this._processPanel.ChooseLibrary(mode);
        }
        
        private void NavPanelProcessView_ViewSwitchedEvent(object sender, JIGSAWMode mode)
        {
            this.SetAppMode(mode);
        }

        private void NavPanelTaskView_ViewSwitchedEvent(object sender, JIGSAWMode mode)
        {
            this.SetAppMode(mode);
        }
        
        private void NavPanelSimulation_ViewSwitchedEvent(object sender, JIGSAWMode mode)
        {
            this.SetAppMode(mode);
        }

        private void MenuItemFileExit_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void Window1_Closing(object sender, CancelEventArgs e)
        {
            this.SimuController.Dispose();
        }

        private void StartClicked(object sender, RoutedEventArgs e)
        {
            //SimuController.Start();
        }

        private void LoadClicked(object sender, RoutedEventArgs e)
        {
            /*
            this.bpmnCanvasHost.Canvas.LoadFile(@"C:\temp\CustomerComplaintProcess.ivn");
             */
        }

        private void OnFileOpenClick(object sender, RoutedEventArgs e)
        {
            App myApp = Application.Current as App;

            if(myApp != null)
            {
                //myApp.BpmnCanvasHost.Canvas.LoadFile(@"C:\Temp\CustomerComplaintProcess.ivn");
            }
        }

        //private void MyTestingMenuItem_Click(object sender, RoutedEventArgs e)
        //{
        //    // Toggle to Simulation View
        //    if(_appMode != JIGSAWMode.Testing)
        //    {
        //        this.SetAppMode(JIGSAWMode.Testing);
        //    }
        //    else
        //    {
        //        this.SetAppMode(JIGSAWMode.ProcessView);
        //    }
        //}
        #endregion

        private void InitPanels()
        {
            ///
            /// Test Panel
            ///
            this._testPanel = this.mainContentBorder.Child;

            ///
            /// Process View Panel
            ///
            this._processPanel = new ProcessViewPane();
            this._processPanel.VerticalAlignment = VerticalAlignment.Stretch;
            this._processPanel.HorizontalAlignment = HorizontalAlignment.Stretch;
            this._processPanel.Width = Double.NaN;
            this._processPanel.Height = Double.NaN;

            ///
            /// Process View Toolbar
            ///
            this._navPanelProcessView = new ProcessView_NavigationPanel();
            this._navPanelProcessView.VerticalAlignment = VerticalAlignment.Stretch;
            this._navPanelProcessView.HorizontalAlignment = HorizontalAlignment.Stretch;
            this._navPanelProcessView.Width = Double.NaN;
            this._navPanelProcessView.Height = Double.NaN;
            this._navPanelProcessView.Visibility = Visibility.Visible;

            this._navPanelProcessView.SetProcessCanvas(this._processPanel.processCanvasHost.Canvas);

            ///
            /// Task View Toolbar
            ///
            this._navPanelTaskView = new TaskView_NavigationPanel();
            this._navPanelTaskView.VerticalAlignment = VerticalAlignment.Stretch;
            this._navPanelTaskView.HorizontalAlignment = HorizontalAlignment.Stretch;
            this._navPanelTaskView.Width = Double.NaN;
            this._navPanelTaskView.Height = Double.NaN;
            this._navPanelTaskView.Visibility = Visibility.Visible;

            this._navPanelTaskView.SetWorkflowCanvas(this._processPanel.workflowCanvasHost.Canvas);

            ///
            /// Simulation Toolbar
            ///
            this._navPanelSimulation = (navigationPanel)this.toolbarBorder.Child;
            this._navPanelSimulation.ToolBar.SetSimuController(this.SimuController);

            ///
            /// Containers
            ///
            toolbarBorder2.BorderBrush = null;
            toolbarBorder2.BorderThickness = new Thickness(0.0d);
            toolbarBorder2.Child = this._navPanelProcessView;

            mainBorder2.BorderBrush = null;
            mainBorder2.BorderThickness = new Thickness(0.0d);
            mainBorder2.Child = this._processPanel;

            //DockPanel canvasPanel = new DockPanel();
            
            //canvasPanel.BeginInit();
            //canvasPanel.VerticalAlignment = VerticalAlignment.Stretch;
            //canvasPanel.HorizontalAlignment = HorizontalAlignment.Stretch;
            //canvasPanel.Width = Double.NaN;
            //canvasPanel.Height = Double.NaN;
            //canvasPanel.Children.Add(this._legendProcess);
            //DockPanel.SetDock(this._legendProcess, Dock.Bottom);
            //canvasPanel.Children.Add(this._processPanel);
            //DockPanel.SetDock(this._processPanel, Dock.Top);
            //canvasPanel.EndInit();

            //mainBorder2.Child = canvasPanel;

            toolbarBorder3.BorderBrush = null;
            toolbarBorder3.BorderThickness = new Thickness(0.0d);
            toolbarBorder3.Child = this._navPanelTaskView;

            ///
            /// Legends
            ///

            // Create source
            //BitmapImage legendBitmap = new BitmapImage();

            //// BitmapImage.UriSource must be in a BeginInit/EndInit block
            //legendBitmap.BeginInit();
            //legendBitmap.UriSource = new Uri(@"graphics\legend-stub.bmp", UriKind.Relative);
            //legendBitmap.DecodePixelHeight = 49;
            //legendBitmap.EndInit();
            
            ////set image height and source
            //this._legendProcess.Height = 49;
            //this._legendProcess.Source = legendBitmap;
        }

        
    }

    public enum JIGSAWMode
    {
        ProcessView,
        WorkflowComposer,
        Testing
    }

    public enum LibraryMode
    {
        ServiceLib,
        UILib,
    }
}