using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using ILOG.Diagrammer;
using System.Drawing;
using ILOG.Diagrammer.Windows.Forms;
using System.Drawing.Drawing2D;
using ILOG.Diagrammer.Graphic;

namespace ProcessView
{
        public class LibraryInteractor : ViewInteractor
        {
            ILOG.Diagrammer.Stroke _stroke = new Stroke(Color.Black, 1, DashStyle.Dot);

            UserSymbol _userSymbol = null;
            Point2D _currentPt = Point2D.Empty;
            bool _isMouseOver = false;
            Bitmap _ghostImage = null;

            public LibraryInteractor(Bitmap ghostImage)
                : this(MouseButtons.Left, ghostImage)
            {
            }

            public LibraryInteractor(MouseButtons button, Bitmap ghostImage)
                : base(button)
            {
                _ghostImage = ghostImage;
            }

            public bool IsMouseOver
            {
                get
                {
                    return _isMouseOver;
                }
            }

            public void Start(UserSymbol userSymbol)
            {
                this._userSymbol = userSymbol;
                if(userSymbol != null)
                {
                    this.StartInteraction();
                }
            }

            public void Stop()
            {
                this.StopInteraction(true);
            }

            public Stroke GhostStroke
            {
                get
                {
                    return _stroke;
                }
                set
                {
                    _stroke = value;
                }
            }

            protected override void OnMouseEnter(EventArgs e)
            {
                this._isMouseOver = true;
                base.OnMouseEnter(e);
            }

            protected override void OnMouseLeave(EventArgs e)
            {
                this._isMouseOver = false;
                base.OnMouseLeave(e);
            }

            

            protected override void OnMouseDown(MouseEventArgs e)
            {
                if(InOperation)
                {
                    if(e.Button == MouseButtons.Left)
                    {
                        DoIt();
                    }

                    StopInteraction(true);
                    this._isMouseOver = false;
                }
                base.OnMouseDown(e);
            }

            protected override void OnMouseMove(MouseEventArgs e)
            {
                if (InOperation && IsMouseOver)
                {
                    /*
                    Rectangle bounds = GetGhostBounds();
                    Point2D currentPt = new Point2D(e.X, e.Y);
                    _radius = GetDistance(_startPt, currentPt);
                    if (bounds.IsEmpty)
                        bounds = GetGhostBounds();
                    else
                        bounds = Rectangle.Union(bounds, GetGhostBounds());
                    InvalidateView(bounds);
                     */
                    _currentPt = new Point2D(e.X, e.Y);

                    this.InvalidateView();
                }
                base.OnMouseMove(e);
            }

            protected override void OnMouseUp(MouseEventArgs e)
            {
                if (InOperation)
                {
                    DoIt();
                    
                    StopInteraction(true);
                    this._isMouseOver = false;
                }
                base.OnMouseUp(e);
            }

            protected override void OnKeyDown(KeyEventArgs e)
            {
                if (e.KeyCode == Keys.Escape)
                {
                    //Rectangle bounds = GetGhostBounds();
                    //View.Invalidate(bounds);
                    StopInteraction(true);

                    InvalidateView();
                }
                base.OnKeyDown(e);
            }
            
            protected System.Drawing.Rectangle GetGhostGeometry()
            {
                Rectangle bounds =
                  new Rectangle((int)(this._currentPt.X - (this._userSymbol.Width / 2)),
                                (int)(this._currentPt.Y - (this._userSymbol.Height / 2)),
                                (int)this._userSymbol.Width,
                                (int)this._userSymbol.Height);
                return bounds;
            }

            protected Rectangle GetGhostBounds()
            {
                Rectangle bounds = GetGhostGeometry();
                using (Pen pen = _stroke.GetPenForBounds(View.Transform))
                {
                    float width = pen.Width;
                    bounds.Inflate((int)width, (int)width);
                    bounds.Offset((int)(-width / 2), (int)(-width / 2));
                }
                return bounds;
            }

            protected override void DrawGhost(ILOG.Diagrammer.DrawingContext context)
            {
                if (InOperation && IsMouseOver)
                {
                    Graphics g = context.Graphics;
                    
                    //Bitmap screenSymbol = global::ProcessView.Properties.Resources.ScreenSymbolPNG;

                    Point2D ghostLoc = new Point2D( (this._currentPt.X - (this._ghostImage.Width / 2)),
                                                    (this._currentPt.Y - (this._ghostImage.Height / 2)));
                    g.DrawImage(this._ghostImage, ghostLoc);
                }
                else
                {
                    base.DrawGhost(context);
                }
                this.InvalidateView();
            }

            protected override void StopInteraction(bool validate)
            {
                _currentPt = Point2D.Empty;
                base.StopInteraction(validate);
            }

            protected virtual void DoIt()
            {
                Rectangle rect = GetGhostGeometry();
                Transform t = View.Content.Transform;
                t = t * View.Transform;
                t = t.Inverse();

                // Computes the points of the rect in the container coordinates
                /*
                Point2D[] points = GetPointsFromRectangle(rect);
                t.TransformPoints(points);
                // Sets the object's size
                if (rect.Width != 0f || rect.Height != 0f)
                {
                    float w = GetDistance(points[0], points[1]);
                    float h = GetDistance(points[1], points[2]);
                    ellipse.Size = new Size2D(w, h);
                }
                */
                // Sets the object's location
                Point2D center = t.TransformPoint(_currentPt);
                this._userSymbol.Move(center, ContentAlignment.MiddleCenter);
                GraphicObject[] obj = new GraphicObject[] { this._userSymbol };
                IManageChildren container = View.Content as IManageChildren;
                if (container != null)
                    container.AddChildren(obj);
                if (SelectOnCreate)
                {
                    View.Selection.SetSelectedObjects(obj);
                }
            }

            static public Point2D[] GetPointsFromRectangle(Rectangle2D rect)
            {
                return new Point2D[] {
                    new Point2D(rect.Left, rect.Top),
                    new Point2D(rect.Right, rect.Top),
                    new Point2D(rect.Right, rect.Bottom),
                    new Point2D(rect.Left, rect.Bottom) };
            }

        }
}
