using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using ILOG.Diagrammer.Windows.Forms;
using ILOG.Diagrammer;
using ILOG.Diagrammer.Graphic;

namespace ProcessView
{
    public delegate void InteractorChangedHandler(object sender, ViewInteractor newInteractor);
    public delegate void SelectionChangedHandler(object sender, bool drillDownAllowed);

    public partial class ProcessCanvas : UserControl
    {
        //public Cursor PanCursor;
        private PanInteractor panInteractor;
        private ZoomInteractor zoomInteractor;
        private ProcessSelectionInteractor selectInteractor;
        private CreateGraphicObjectInteractor inter;
        private CreateLinkInteractor ektaInteractor;

        public event DrillDownHandler DrillDownEvent;
        public event InteractorChangedHandler InteractorChangedEvent;
        public event InteractorChangedHandler InteractorResetEvent;
        public event SelectionChangedHandler SelectionChangedEvent;

        private void OnSelectionChanged(bool isDrillDownAllowed)
        {
            if(SelectionChangedEvent != null)
            {
                SelectionChangedEvent(this, isDrillDownAllowed);
            }
        }

        private void OnDrillDown(DrillDownEventArgs args)
        {
            if(DrillDownEvent != null)
            {
                DrillDownEvent(this, args);
            }
        }

        private void OnInteractorChanged(ViewInteractor newInteractor)
        {
            if(InteractorChangedEvent != null)
            {
                InteractorChangedEvent(this, newInteractor);
            }
        }

        private void OnInteractorReset(ViewInteractor newInteractor)
        {
            if(InteractorResetEvent != null)
            {
                InteractorResetEvent(this, newInteractor);
            }
        }

        public ProcessCanvas()
        {
            InitializeComponent();

            this.panInteractor = new ILOG.Diagrammer.Windows.Forms.PanInteractor(MouseButtons.Left);
            this.zoomInteractor = new ILOG.Diagrammer.Windows.Forms.ZoomInteractor();
            this.selectInteractor = new ProcessSelectionInteractor(processDiagram1);
            this.selectInteractor.DrillDownEvent += new DrillDownHandler(selectInteractor_DrillDownEvent);
            
            this.inter = new CreateGraphicObjectInteractor();
            this.inter.ValidateOperation += new EventHandler(inter_ValidateOperation);
            //this.inter.ObjectCreated += new GraphicObjectCreatedEventHandler(inter_ObjectCreated);
            
            this.ektaInteractor = new CreateLinkInteractor();
            this.ektaInteractor.ObjectCreated += new GraphicObjectCreatedEventHandler(OnObjectCreated);
            this.ektaInteractor.ValidateOperation += new EventHandler(ektaInteractor_ValidateOperation);

            this.diagramView1.Content = this.processDiagram1;
            this.diagramView1.Interactor = this.selectInteractor;

            //this.isSelectedAddTask = false;
            //this.isSelectedAddConnection = false;
            //this.isSelectedPan = false;
            //this.isSelectedZoomIn = false;
            //this.isSelectedZoomOut = false;
            this.processDiagram1.DrillDownEvent += new DrillDownHandler(processDiagram1_DrillDownEvent);
            this.diagramView1.InteractorChanged += new EventHandler(diagramView1_InteractorChanged);
            
            //this.diagramView1.SelectionChanged += new SelectionChangedEventHandler(diagramView1_SelectionChanged);
        }

        /*
        void diagramView1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            GraphicObject[] selectedObjects = this.diagramView1.Selection.GetSelectedObjects();

            int numberOfSystemTasksSelected = 0;
            GraphicObject selectedSystemTask = null;

            foreach(GraphicObject go in selectedObjects)
            {
                if(this.processDiagram1.IsDiagramItem(go))
                {
                    numberOfSystemTasksSelected++;
                    selectedSystemTask = go;
                }
            }

            if(numberOfSystemTasksSelected == 1 && (selectedSystemTask != null))
            {
                OnSelectionChanged(true);
            }
            else
            {
                if(this.HasDrillDownSelection)
                {
                    OnSelectionChanged(true);
                }
                else
                {
                    OnSelectionChanged(false);
                }
            }
        }
        */

        void selectInteractor_DrillDownEvent(object sender, DrillDownEventArgs args)
        {
            OnDrillDown(args);
        }

        void diagramView1_InteractorChanged(object sender, EventArgs e)
        {
            if((this.diagramView1.Interactor != null) && (this.diagramView1.Interactor is SelectInteractor) && (!(this.diagramView1.Interactor is ProcessSelectionInteractor)))
            {
                DefaultToSelect();
            }
        }

        void processDiagram1_DrillDownEvent(object sender, DrillDownEventArgs args)
        {
            OnDrillDown(args);
        }

        public void ShowAll()
        {
            this.diagramView1.ShowAll();
            //this.diagramView1.Refresh();
        }

        public void ZoomOut()
        {
            this.diagramView1.ZoomOut();
        }

        /*
        public void ShowAll(int width, int height)
        {
            this.Width = width;
            this.Height = height;

            this.diagramView1.ShowAll();
            this.diagramView1.Refresh();
        }
         */

        
        public void ClearSelection()
        {
            this.processDiagram1.RemoveSelection();
        }

        public bool ReadOnly
        {
            get
            {
                return this.processDiagram1.ReadOnly;
            }
            set
            {
                if(value)
                {
                    this.diagramView1.SelectionStyle.CanConnect = false;
                    this.diagramView1.SelectionStyle.CanCopy = false;
                    this.diagramView1.SelectionStyle.CanEditShape = false;
                    this.diagramView1.SelectionStyle.CanEditText = false;
                    this.diagramView1.SelectionStyle.CanMove = false;
                    this.diagramView1.SelectionStyle.CanReparent = false;
                    this.diagramView1.SelectionStyle.CanResize = false;
                    this.diagramView1.SelectionStyle.CanRotate = false;
                    
                }
                else
                {
                    this.diagramView1.SelectionStyle.CanConnect = true;
                    this.diagramView1.SelectionStyle.CanCopy = true;
                    this.diagramView1.SelectionStyle.CanEditShape = true;
                    this.diagramView1.SelectionStyle.CanEditText = true;
                    this.diagramView1.SelectionStyle.CanMove = true;
                    this.diagramView1.SelectionStyle.CanReparent = true;
                    this.diagramView1.SelectionStyle.CanResize = true;
                    this.diagramView1.SelectionStyle.CanRotate = true;
                }
                
                this.processDiagram1.ReadOnly = value;
            }
        }

        private void inter_ObjectCreated(object sender, GraphicObjectCreatedEventArgs e)
        {
            //Not used momentarily...
        }

        private void ektaInteractor_ValidateOperation(object sender, EventArgs e)
        {
            this.DefaultToSelect();
        }

        private void inter_ValidateOperation(object sender, EventArgs e)
        {
            this.DefaultToSelect();
        }

        /*
        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            this.InteractorSelectionMode(CanvasInteractionMode.ZoomIn, false);
            //this.diagramView1.ZoomIn();
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            this.InteractorSelectionMode(CanvasInteractionMode.ZoomOut, false);
            //this.diagramView1.ZoomOut();

        }

        private void toolStripButton3_Click(object sender, EventArgs e)
        {

            //this.diagramView1.Interactor = panInteractor;
            this.InteractorSelectionMode(CanvasInteractionMode.Pan, false);         

        }

        private void AddTasktoolStripButton_Click(object sender, EventArgs e)
        {
            //inter.Factory = CreateGraphicObjectInteractor.RectFactory;
            //this.diagramView1.Interactor = inter;
            this.InteractorSelectionMode(CanvasInteractionMode.AddTask, false);
        }

        private void ConnectiontoolStripButton_Click(object sender, EventArgs e)
        {
            //diagramView1.Interactor = this.ektaInteractor;
            this.InteractorSelectionMode(CanvasInteractionMode.AddConnection, false);
        }
        */

        private void DefaultToSelect()
        {
            this.diagramView1.Interactor = this.selectInteractor;

            OnInteractorChanged(this.selectInteractor);
            OnInteractorReset(this.selectInteractor);
        }

        private void OnObjectCreated(object sender, GraphicObjectCreatedEventArgs e)
        {
            Link link = e.GraphicObject as Link;
            link.ShapeType = LinkShapeType.Orthogonal;
            link.CrossingStyle = LinkCrossingsStyle.Bridge;
            link.CanMove = true;
        }

        public void InteractorSelectionMode(CanvasInteractionMode modeOfInteraction, bool selectToggleState)
        {
            switch (modeOfInteraction)
            {
                case CanvasInteractionMode.None:
                    this.DoNoneSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.Pan:
                    this.DoPanSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.ZoomIn:
                    this.DoZoomInSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.ZoomOut:
                    this.DoZoomOutSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.AddTask:
                    this.DoAddTaskSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.AddConnection:
                    this.DoAddConnectionSelection(selectToggleState);                    
                    break;
                case CanvasInteractionMode.AddSystemTask:
                    this.DoAddSystemTaskSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.AddDecision:
                    this.DoAddDecisionSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.AddStartPoint:
                    this.DoAddStartPointSelection(selectToggleState);
                    break;
                case CanvasInteractionMode.AddEndPoint:
                    this.DoAddEndPointSelection(selectToggleState);
                    break;
            }
        }

        private void DoNoneSelection(bool isSelected)
        {
            this.DefaultToSelect();
        }
        private void DoPanSelection(bool isSelected)
        {
            if (isSelected)
            {
                this.DefaultToSelect();
            }
            else
                this.diagramView1.Interactor = panInteractor;
        }
        private void DoZoomInSelection(bool isSelected)
        {
            if(isSelected)
            {
                this.DefaultToSelect();
            }
            else
                this.diagramView1.ZoomIn();
        }
        private void DoZoomOutSelection(bool isSelected)
        {
            if (isSelected)
            {
                this.DefaultToSelect();
            }
            else
                this.diagramView1.ZoomOut();
        }
        private void DoAddTaskSelection(bool isSelected)
        {
            if (isSelected)
            {
                this.DefaultToSelect();
            }
            else
            {
                inter.Factory = new ManualTaskUserSymbolFactory();
                //inter.Factory = CreateGraphicObjectInteractor.RectFactory;
                //inter.RectangleColor = Color.White;
                this.diagramView1.Interactor = inter;
            }
        }

        private void DoAddSystemTaskSelection(bool isSelected)
        {
            if (isSelected)
            {
                this.DefaultToSelect();
            }
            else
            {
                inter.Factory = new SystemTaskUserSymbolFactory();
                //inter.Factory = CreateGraphicObjectInteractor.RectFactory;
                //inter.RectangleColor = Color.White;
                this.diagramView1.Interactor = inter;
            }
        }

        private void DoAddConnectionSelection(bool isSelected)
        {
            if (isSelected)
            {
                this.DefaultToSelect();
            }
            else
                diagramView1.Interactor = this.ektaInteractor;
        }

        private void DoAddDecisionSelection(bool isSelected)
        {
            if (isSelected)
            {
                this.DefaultToSelect();
            }
            else
            {
                inter.Factory = new DecisionUserSymbolFactory();
                //inter.Factory = CreateGraphicObjectInteractor.RectFactory;
                //inter.RectangleColor = Color.White;
                this.diagramView1.Interactor = inter;
            }
        }

        private void DoAddEndPointSelection(bool isSelected)
        {
            if(isSelected)
            {
                this.DefaultToSelect();
            }
            else
            {
                inter.Factory = new EndPointSymbolFactory();
                this.diagramView1.Interactor = inter;
            }
        }

        private void DoAddStartPointSelection(bool isSelected)
        {
            if(isSelected)
            {
                this.DefaultToSelect();
            }
            else
            {
                inter.Factory = new StartPointSymbolFactory();
                this.diagramView1.Interactor = inter;
            }
        }

        public bool ShowLegend
        {
            set
            {
                this.panel4.Visible = value;
                this.overview1.Visible = value;
            }
        }

        public void DeleteSelection()
        {
            GraphicObject[] graphicObjects = this.diagramView1.Selection.GetSelectedObjects();

            foreach(GraphicObject go in graphicObjects)
            {
                go.RemoveFromParent();
            }
        }

        public void DrillDownSelectedTask()
        {
            GraphicObject[] selectedObjects = this.diagramView1.Selection.GetSelectedObjects();

            int numberOfSystemTasksSelected = 0;
            GraphicObject selectedSystemTask = null;

            foreach(GraphicObject go in selectedObjects)
            {
                if(this.processDiagram1.IsDiagramItem(go))
                {
                    numberOfSystemTasksSelected++;
                    selectedSystemTask = go;
                }
            }

            if(numberOfSystemTasksSelected == 1 && (selectedSystemTask != null))
            {
                this.processDiagram1.Select(selectedSystemTask);
            }
        }

        /*
        public bool HasDrillDownSelection
        {
            get
            {
                return this.processDiagram1.HasDrillDownSelection;
            }
        }
         */

        public enum CanvasInteractionMode
        {
            None,
            Pan,
            ZoomIn,
            ZoomOut,
            AddTask,
            AddSystemTask,
            AddDecision,
            AddConnection,
            AddStartPoint,
            AddEndPoint
        }
    }
}
