using System;
using System.Collections.Generic;
using System.Text;

using ILOG.Diagrammer;
using ILOG.Diagrammer.Windows.Forms;
using System.Windows.Forms;
using ILOG.Diagrammer.Graphic;

namespace ProcessView
{
    public class ProcessSelectionInteractor: SelectInteractor
    {
        private ProcessDiagram _diagram = null;

        private GraphicObject _currentDrillTarget = null;

        ContextMenu _ghost = null;
        MenuItem _drillItem = null;
        MenuItem _delItem = null;

        public event DrillDownHandler DrillDownEvent;

        private void OnDrillDown(GraphicObject go)
        {
            if(DrillDownEvent != null)
            {
                DrillDownEvent(this, new DrillDownEventArgs(go));
            }
        }

        public ProcessSelectionInteractor(ProcessDiagram diagram)
            :base()
        {
            _diagram = diagram;

            _drillItem = new MenuItem("Edit");
            _delItem = new MenuItem("Delete");
            _drillItem.Click +=new EventHandler(drillItem_Click);
            _delItem.Click += new EventHandler(delItem_Click);
            _ghost = new ContextMenu(new MenuItem[] { _drillItem, _delItem });
            _ghost.Collapse += new EventHandler(_ghost_Collapse);
        }

        public ProcessDiagram Diagram
        {
            get
            {
                return _diagram;
            }
            set
            {
                _diagram = value;
            }
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            if((e.Button == MouseButtons.Right) && (!this.Diagram.ReadOnly))
            {
                GraphicObject go = this.View.HitTestSelectable(e.Location);

                this._drillItem.Enabled = false;
                
                if((go != null) && _diagram.IsDiagramItem(go))
                {
                    this._drillItem.Enabled = true;    
                }

                _ghost.Show(this.View, e.Location);

                _currentDrillTarget = go;
            }
            base.OnMouseDown(e);
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if((e.KeyCode == Keys.Delete) && (!this.Diagram.ReadOnly))
            {
                this.Diagram.Delete(this.Selection.GetSelectedObjects());
            }

            base.OnKeyDown(e);
        }

        private void drillItem_Click(object sender, EventArgs e)
        {
            if(_currentDrillTarget != null)
            {
                this.Diagram.AddSelection(_currentDrillTarget);
                OnDrillDown(_currentDrillTarget);
            }
        }

        private void delItem_Click(object sender, EventArgs e)
        {
            if(_currentDrillTarget != null)
            {
                this.Diagram.Delete(new GraphicObject[] { _currentDrillTarget });
            }
        }

        private void _ghost_Collapse(object sender, EventArgs e)
        {
            _currentDrillTarget = null;
        }
    }
}
