using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

namespace SimulationEngine
{

    public sealed class SimulationDefinition
    {
        private ArrayList Screens = new ArrayList();
        private ArrayList Actions = new ArrayList();

        private ScreenDef _startScreen = null;
        public ScreenDef StartScreen
        {
            get
            {
                return _startScreen;
            }
        }

        public static SimulationDefinition instance = null;
        public static readonly object padlock = new object();

        private SimulationDefinition()
        {
            Init();
        }

        public static SimulationDefinition Instance
        {
            get
            {
                lock (padlock)
                {
                    if (instance==null)
                    {
                        instance = new SimulationDefinition();
                    }
                    return instance;
                }
            }
        }


        private void Init()
        {
            ScreenDef screen1 = new ScreenDef("screen1");
            ActionDef action1 = new ActionDef("btCancelScreen1", "screen1");
            ActionDef action2 = new ActionDef("btOKScreen1", "screen2");
            screen1.Actions.Add(action1);
            screen1.Actions.Add(action2);

            ScreenDef screen2 = new ScreenDef("screen2");
            ActionDef action3 = new ActionDef("btSelectScreen2", "screen3");
            screen2.Actions.Add(action3);

            ScreenDef screen3 = new ScreenDef("screen3");
            ActionDef action4 = new ActionDef("btAcceptScreen3", "screen6");
            ActionDef action5 = new ActionDef("btAskScreen3", "screen4");
            ActionDef action6 = new ActionDef("btRejectScreen3", "screen5");
            screen3.Actions.Add(action4);
            screen3.Actions.Add(action5);
            screen3.Actions.Add(action6);

            ScreenDef screen4 = new ScreenDef("screen4");
            ActionDef action7 = new ActionDef("btBackScreen4", "screen3");
            ActionDef action8 = new ActionDef("btForwardScreen4", "done");
            screen4.Actions.Add(action7);
            screen4.Actions.Add(action8);

            ScreenDef screen5 = new ScreenDef("screen5");
            ActionDef action9 = new ActionDef("btBackScreen5", "screen3");
            ActionDef action10 = new ActionDef("btSendScreen5", "done");
            screen5.Actions.Add(action9);
            screen5.Actions.Add(action10);

            ScreenDef screen6 = new ScreenDef("screen6");
            ActionDef action11 = new ActionDef("btBackScreen6", "screen3");
            ActionDef action12 = new ActionDef("btForwardScreen6", "done");
            screen6.Actions.Add(action11);
            screen6.Actions.Add(action12);

            _startScreen = screen1;
        }

        public void Register(ActionDef a)
        {
            if(a != null) this.Actions.Add(a);
        }

        public void Register(ScreenDef s)
        {
            if(s != null) this.Screens.Add(s);
        }

        public ActionDef GetActionDefByName(string name)
        {
            ActionDef theAction = null;
            
            foreach(ActionDef action in this.Actions)
            {
                if(action.ActionId.Equals(name))
                {
                    theAction = action;
                    break;
                }
            }

            return theAction;
        }

        public ScreenDef GetScreenDefByName(string name)
        {
            ScreenDef theScreen = null;

            foreach(ScreenDef screen in this.Screens)
            {
                if(screen.ScreenId.Equals(name))
                {
                    theScreen = screen;
                    break;
                }
            }

            return theScreen;
        }
    }
}
