﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Controls.Primitives;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace ViewSlider
{
	public partial class UserControl1
	{
        private SliderPos _originalSliderPos = SliderPos.None;

        public delegate void SliderChangedHandler(object sender, SliderPos sliderPos);
        public event SliderChangedHandler SliderChangedEvent;
        private void OnSliderChanged(SliderPos sliderPos)
        {
            if(SliderChangedEvent != null)
            {
                SliderChangedEvent(this.sliderNavigator, sliderPos);
            }
        }

		public UserControl1()
		{
			this.InitializeComponent();

			// Insert code required on object creation below this point.

            this.sliderNavigator.SelectionStart = 1d;
            this.sliderNavigator.SelectionEnd = 3d;

            this.imgTest.MouseUp += new System.Windows.Input.MouseButtonEventHandler(imgTest_MouseUp);
            this.imgProcess.MouseUp += new System.Windows.Input.MouseButtonEventHandler(imgProcess_MouseUp);
            this.imgTask.MouseUp += new System.Windows.Input.MouseButtonEventHandler(imgTask_MouseUp);

            this.sliderNavigator.MouseUp += new System.Windows.Input.MouseButtonEventHandler(sliderNavigator_MouseUp);

            this.Loaded += new RoutedEventHandler(UserControl1_Loaded);

            this.sliderNavigator.ValueChanged += new RoutedPropertyChangedEventHandler<double>(sliderNavigator_ValueChanged);

		}

        void sliderNavigator_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if(e.NewValue < 1.5)
            {
                //Process
                SliderPos = SliderPos.ProcessPos;
            }
            else if((e.NewValue >= 1.5) && (e.NewValue < 2.5))
            {
                //Workflow
                SliderPos = SliderPos.WorkflowPos;
            }
            else
            {
                //Testing
                SliderPos = SliderPos.TestPos;
            }
        }

        private void UserControl1_Loaded(object sender, RoutedEventArgs e)
        {
            // Reset the slider knob
            this.sliderNavigator.Value = (double)this._originalSliderPos;

            ///
            /// Attaching the MouseUp event handler to the custom slider knob (a rounded translucent rectangle)
            ///

            /*
            this.sliderNavigator.Template.LoadContent();
            object thumb = this.sliderNavigator.Template.FindName("Thumb", this.sliderNavigator);
            object knob = this.sliderNavigator.FindName("sliderKnob");

            if(thumb is Thumb)
            {
                Thumb myThumb = (Thumb)thumb;
                myThumb.Template.LoadContent();
                object rect = myThumb.Template.FindName("sliderKnob", myThumb);
                if(rect is Rectangle)
                {
                    Rectangle sliderKnob = (Rectangle)rect;
                    sliderKnob.PreviewMouseUp += new System.Windows.Input.MouseButtonEventHandler(sliderKnob_PreviewMouseUp);
                }
            }
            */
        }

        //void sliderKnob_PreviewMouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        //{
        //    throw new Exception("The method or operation is not implemented.");
        //}

        public SliderPos OriginalSliderPos
        {
            set
            {
                this._originalSliderPos = value;
            }
        }

        private void sliderNavigator_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if(this.sliderNavigator.Value < 1.5)
            {
                //Process
                SliderPos = SliderPos.ProcessPos;
            }
            else if((this.sliderNavigator.Value >= 1.5) && (this.sliderNavigator.Value < 2.5))
            {
                //Workflow
                SliderPos = SliderPos.WorkflowPos;
            }
            else
            {
                //Testing
                SliderPos = SliderPos.TestPos;
            }
        }

        private void imgTask_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.SliderPos = SliderPos.WorkflowPos;
        }

        private void imgProcess_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.SliderPos = SliderPos.ProcessPos;
        }

        private void imgTest_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.SliderPos = SliderPos.TestPos;
        }

        public Image ImgProcess
        {
            get
            {
                return this.imgProcess;
            }
        }
        public Image ImgTask
        {
            get
            {
                return this.imgTask;
            }
        }
        public Image ImgTest
        {
            get
            {
                return this.imgTest;
            }
        }

        public SliderPos SliderPos
        {
            set
            {
                this.sliderNavigator.Value = (double)value;

                OnSliderChanged(this.SliderPos);
            }
            get
            {
                SliderPos currentSliderPos;

                if(this.sliderNavigator.Value < 1.5)
                {
                    //Process
                    currentSliderPos = SliderPos.ProcessPos;
                }
                else if((this.sliderNavigator.Value >= 1.5) && (this.sliderNavigator.Value < 2.5))
                {
                    //Workflow
                    currentSliderPos = SliderPos.WorkflowPos;
                }
                else
                {
                    //Testing
                    currentSliderPos = SliderPos.TestPos;
                }
                
                return currentSliderPos;
            }
        }

        private bool _enabled = true;
        public bool Enabled
        {
            get
            {
                return _enabled;
            }
            set
            {
                //this.sliderNavigator.IsEnabled = value;
                //this.imgTask.IsEnabled = value;
                //this.imgTest.IsEnabled = value;

                //_enabled = value;
            }
        }
	}

    public enum SliderPos
    {
        None,
        ProcessPos,
        WorkflowPos,
        TestPos
    }
}