using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows;
using System.Text.RegularExpressions;

namespace WorkflowLibraryTemplates
{
	public class LibraryController
	{
        private UserControl _library = null;

        private ToggleButton _currentToggleButton;

        private Border _previewHost = null;

        private WrapPanel _wrapPanel = null;

        private List<ToggleButton> _visibleToggleButtons = new List<ToggleButton>();

        private UserControl[] _previews;

        public LibraryController(UserControl library, UserControl[] previews, Border previewHost, WrapPanel wrapPanel)
        {
            _library = library;
            _previews = previews;
            _previewHost = previewHost;
            _wrapPanel = wrapPanel;
        }

        public void HideAll()
        {
            foreach(ToggleButton tbt in _visibleToggleButtons)
            {
                tbt.Visibility = Visibility.Hidden;
            }

            _visibleToggleButtons.Clear();
            _wrapPanel.Children.Clear();
        }

        public void Show(ToggleButton button)
        {
            button.Visibility = Visibility.Visible;
            _wrapPanel.Children.Add(button);
            _visibleToggleButtons.Add(button);
        }

        public void CheckCurrent()
        {
            if(_currentToggleButton != null) _currentToggleButton.IsChecked = true;
            SetPreview(_currentToggleButton);
        }

        private void SetPreview(ToggleButton button)
        {
            if((button != null) && (button.Name != null))
            {
                // Capture the ending number of the button name
                Match m = Regex.Match(button.Name, @"\d{1,2}$");

                if(m.Success)
                {
                    // Retrieve appropriate preview and place into user control
                    int previewIndex = Int32.Parse(m.Value) - 1;

                    if((previewIndex >= 0) && (previewIndex < _previews.Length))
                    {
                        _previewHost.Child = (UserControl)_previews.GetValue(previewIndex);

                        _library.InvalidateArrange();
                        _library.UpdateLayout();
                    }
                }
            }
        }

        public void ThumbToggleButton_Click(object sender, RoutedEventArgs e)
        {
            if(_currentToggleButton != null)
            {
                if(_currentToggleButton == sender as ToggleButton)
                {
                    _currentToggleButton.IsChecked = true;
                }
                else
                {
                    _currentToggleButton.IsChecked = false;
                }
            }

            _currentToggleButton = sender as ToggleButton;

            this.CheckCurrent();
        }
	}
}
